"use strict";
/**
 * @prettier
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.AbstractFetchClient = void 0;
const tslib_1 = require("tslib");
const qs_1 = tslib_1.__importDefault(require("qs"));
class AbstractFetchClient {
    basePath;
    defaultHeaders;
    defaultTimeout;
    constructor(config) {
        this.basePath = config.basePath;
        this.defaultHeaders = config.defaultHeaders;
        this.defaultTimeout = config.defaultTimeout;
    }
    async _fetch(url, opts, timeout = this.defaultTimeout) {
        // main abort controller that will be returned to the caller
        const cancelRequest = new AbortController();
        // if a timeout is provided, set a timeout signal and connect to main abort controller
        if (timeout && timeout > 0) {
            const timeoutSignal = AbortSignal.timeout(timeout);
            timeoutSignal.addEventListener("abort", () => {
                cancelRequest.abort(timeoutSignal.reason);
            });
        }
        // if we were provided a signal connect it to the main abort controller
        const userSignal = opts.signal;
        if (userSignal) {
            userSignal.addEventListener("abort", () => cancelRequest.abort(userSignal.reason));
        }
        const headers = opts.headers ?? this._headers({});
        return fetch(url, {
            ...opts,
            headers,
            signal: cancelRequest.signal,
        }).catch((err) => {
            // workaround bug where eg: TimeoutError gets converted to an AbortError
            // https://stackoverflow.com/a/75973817/1445636
            if (err?.name === "AbortError") {
                cancelRequest.signal.throwIfAborted();
            }
            // if not aborted just throw
            throw err;
        });
    }
    _query(params) {
        const definedParams = Object.entries(params).filter(([, v]) => v !== undefined);
        if (!definedParams.length) {
            return "";
        }
        return ("?" + qs_1.default.stringify(Object.fromEntries(definedParams), { indices: false }));
    }
    _headers(headers) {
        return Object.fromEntries(Object.entries({ ...this.defaultHeaders, ...headers }).filter((it) => it[1] !== undefined));
    }
}
exports.AbstractFetchClient = AbstractFetchClient;
//# sourceMappingURL=main.js.map